<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Helper\Catalog\ProductHelper;
use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use FiloBlu\Refilo\Remote\Entity\Product;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product\Attribute\Source\Status as AttributeStatus;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class DisabledProductProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
class DisabledProductProvider extends AbstractFromCollectionProvider
{
    /**
     * @var CollectionFactory
     */
    private $productCollectionFactory;
    /**
     * @var WebsiteInterface
     */
    private $website;
    /**
     * @var StoreInterface
     */
    private $store;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var ProductHelper
     */
    private $productHelper;

    /**
     * @var
     */
    private $productCollection;


    /**
     * DisabledProductProvider constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param StoreManagerInterface $storeManager
     * @param ProductHelper $productHelper
     * @param CollectionFactory $productCollectionFactory
     */
    public function __construct(ScopeConfigInterface $scopeConfig,
                                StoreManagerInterface $storeManager,
                                ProductHelper $productHelper,
                                CollectionFactory $productCollectionFactory)
    {
        parent::__construct($scopeConfig);
        $this->productCollectionFactory = $productCollectionFactory;
        $this->storeManager = $storeManager;
        $this->productHelper = $productHelper;
    }

    /**
     * @return Collection
     */
    public function getCollection()
    {
        if ($this->productCollection === null) {
            $this->productCollection = $this->productCollectionFactory->create()
                ->addAttributeToSelect('status')
                ->addAttributeToSelect('name');
        }

        return $this->productCollection;
    }

    /**
     * @param $items
     * @return array
     */
    public function adapt($items)
    {
        $output = [];
        /** @var ProductInterface $magentoProduct */
        foreach ($items as $magentoProduct) {

            if ((int)$magentoProduct->getStatus() === AttributeStatus::STATUS_DISABLED) {
                $output[$magentoProduct->getEntityId()] = new Product([
                    Product::DOCTYPE    => Product::DOCTYPE_SPEC_DISABLED,
                    Product::ID         => (int)$magentoProduct->getEntityId(),
                    Product::ENABLED    => '0',
                    Product::SKU        => $magentoProduct->getSku(),
                    Product::TYPE       => $magentoProduct->getTypeId(),
                    Product::NAME       => $magentoProduct->getName(),
                    Product::CREATED_AT => $magentoProduct->getCreatedAt(),
                    Product::UPDATED_AT => $magentoProduct->getUpdatedAt()
                ]);
                continue;
            }

            if (!$this->website) {
                continue;
            }

            if (!$this->productHelper->productBelongsTo($magentoProduct->getEntityId(), $this->website->getId())) {
                $output[$magentoProduct->getEntityId()] = new Product([
                    Product::DOCTYPE    => Product::DOCTYPE_SPEC_DISABLED,
                    Product::ID         => (int)$magentoProduct->getEntityId(),
                    Product::ENABLED    => '0',
                    Product::SKU        => $magentoProduct->getSku(),
                    Product::TYPE       => $magentoProduct->getTypeId(),
                    Product::NAME       => $magentoProduct->getName(),
                    Product::CREATED_AT => $magentoProduct->getCreatedAt(),
                    Product::UPDATED_AT => $magentoProduct->getUpdatedAt()
                ]);
                continue;
            }

            $output[$magentoProduct->getEntityId()] = new Product([
                Product::ID => (int)$magentoProduct->getEntityId()
            ]);

        }

        return $output;
    }

    /**
     * @param StoreInterface $store
     * @return EntityProviderInterface
     * @throws LocalizedException
     */
    public function forStore(StoreInterface $store): EntityProviderInterface
    {
        $this->store = $store;
        $this->website = $this->storeManager->getWebsite($store->getWebsiteId());
        return $this;
    }

    /**
     * @param WebsiteInterface $website
     * @return EntityProviderInterface
     */
    public function forWebsite(WebsiteInterface $website): EntityProviderInterface
    {
        $this->website = $website;
        return $this;
    }

}
